/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.uima.camel;

import java.net.URI;
import java.util.Map;

import org.apache.camel.Endpoint;
import org.apache.camel.Exchange;
import org.apache.camel.RuntimeCamelException;
import org.apache.camel.impl.DefaultComponent;

/**
 * Defines the UIMA-AS camel driver component.
 */
public class UimaAsComponent extends DefaultComponent<Exchange> {

	@Override
	protected Endpoint<Exchange> createEndpoint(String uri, String remaining,
    		Map parameters) throws Exception {
		
    	URI hbaseURI = URI.create(uri);
    	
    	String scheme = hbaseURI.getScheme();
    	
    	if (!"uimadriver".equals(scheme)) {
    		throw new RuntimeCamelException("Unsupported scheme: " + scheme);
    	}
    	
    	// get string between first : and last ?
    	int indexBeginBrokerUrl = uri.indexOf(':');
    	int indexEndBrokerUrl = uri.lastIndexOf('?');
    	
    	if (indexBeginBrokerUrl == -1 || indexEndBrokerUrl == -1 ||
    			indexBeginBrokerUrl + 1 == uri.length()) {
    		throw new RuntimeCamelException("Invalid broker Url!");
    	}
    	
    	indexBeginBrokerUrl += 1;
    	
    	String brokerAddress = uri.substring(indexBeginBrokerUrl,
    			indexEndBrokerUrl);
		
    	return new UimaAsEndpoint(uri, brokerAddress, this);
	}
}
